#!/usr/local/bin/ruby
# $Id: setup.rb,v 1.12 2000/08/19 14:03:22 tommy Exp $

DataFiles = ['conf-dist', 'entry', 'w3ml.rb', 'umail.rb', 'w3ml_mysql.rb']
CgiFiles = ['index.cgi', 'list.cgi', 'thread.cgi', 'msg.cgi', 'attach.cgi', 'search.cgi', 'raw.cgi']
CssFiles = ['w3ml.css']

def ruby_path()
  ENV['PATH'].split(/:/).each do |p|
    if File::executable? p+'/ruby' then
      return p+'/ruby'
    end
  end
  nil
end

def cgidir()
  ['/usr/local/apache/cgi-bin', '/home/httpd/cgi-bin'].each do |d|
    return d if File::directory? d
  end
  nil
end

def htmldir()
  ['/usr/local/apache/htdocs', '/home/httpd/html'].each do |d|
    return d if File::directory? d
  end
  nil
end

$params = {}

if File::exist? 'setup.cache' then
  File::open('setup.cache').each do |r|
    a = r.split /\t/, 2
    $params[a[0]] = a[1].chomp
  end
end

def prompt(str, var, defs)
  begin
    $params[var] ||= defs
    print "#{str} [#{$params[var]}] "
    $stdout.flush
    r = $stdin.gets.chomp
    if r != "" then
      $params[var] = r
    end
  end while $params[var] == nil
end

def mkdir_p(dir)
  a = dir.split(/\//)
  2.upto(a.length) do |i|
    d = a[0,i].join('/')
    if not File::exist? d then Dir::mkdir d end
  end
end

def setup()
  prompt 'ML name', 'ML', nil
  prompt 'Ruby path', 'RUBY_PATH', ruby_path
  prompt 'Data directory', 'DATA_DIR', "/usr/local/w3ml/#{$params['ML']}"
  prompt 'CGI directory', 'CGI_DIR', "#{cgidir}/#{$params['ML']}"
  prompt 'CSS directory (Real Path)', 'CSS_DIR', "#{htmldir}/css"
  prompt 'CSS directory (Virtual Path (URL))', 'CSS_URL', '/css'
  prompt 'Use MySQL?', 'UseMysql', 'no'
  if $params['UseMysql'] =~ /^y/i then
    prompt 'MySQL server hostname', 'MysqlHost', 'localhost'
    prompt 'MySQL username', 'MysqlUser', nil
    prompt 'MySQL password', 'MysqlPass', nil
    prompt 'MySQL Database', 'MysqlDB', "w3ml_#{$params['ML']}"
  end

  File::open('setup.cache', 'w') do |f|
    $params.each do |k, v|
      f.puts "#{k}\t#{v}"
    end
  end

  (DataFiles+CgiFiles+CssFiles).each do |f|
    if File::exist? "#{f}.in" then
      w = File::open(f, "w")
      r = File::open("#{f}.in")
      w.write r.read.gsub(/@(\w+)@/) do |s|
	$params[$1] || s
      end
      r.close
      w.close
    end
  end
  system "chmod +x entry *.cgi"
end

def install()
  if not File::exist? 'conf-dist' or not File::exist? 'entry' then
    setup
  end

  mkdir_p $params['DATA_DIR'] if not File::directory? $params['DATA_DIR']
  mkdir_p $params['CGI_DIR'] if not File::directory? $params['CGI_DIR']
  mkdir_p $params['CSS_DIR'] if not File::directory? $params['CSS_DIR']

  system "cp -f #{DataFiles.join ' '} #{$params['DATA_DIR']}"
  system "cp -f #{CgiFiles.join ' '} #{$params['CGI_DIR']}"
  system "cp -f #{CssFiles.join ' '} #{$params['CSS_DIR']}"
  system "cp -f #{$params['DATA_DIR']}/w3ml_mysql.rb #{$params['DATA_DIR']}/w3ml.rb" if $params['UseMysql'] =~ /^y/i
  system "cp #{$params['DATA_DIR']}/conf-dist #{$params['DATA_DIR']}/conf" if not File::exist? "#{$params['DATA_DIR']}/conf"
end

def usage()
  puts 'usage: ruby setup.rb {setup|install}'
  exit 1
end

if ARGV.length != 1 then
  usage
end

case ARGV[0]
when 'setup' then setup
when 'install' then
  install
  if $params['UseMysql'] =~ /^y/ then
    puts
    puts "Please create database `#{$params['MysqlDB']}' on MySQL"
    puts "and create tables using `db.sql' file."
    puts
  end
else
  usage
end
